#!/usr/bin/python

#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are
#  met:
#  
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following disclaimer
#    in the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of the  nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#  
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#  


# This is the main application (hence name App.py) for the project
# called "Speaking Assitant" (It is named so for the lack of creativity)
# This app is tested on Intel Galileo Gen2.
# The project uses an IR proximity sensor as human machine interface.
# By detecting the number of times the IR sensor detects the obstruction, it takes a predefined
# "operation" as dictated by the file "configuration.py"
# The "operation" could be as simple as trurning a LED on or off or it could be fetching RSS
# feed and reading it out using TTS.

import TTSHandler           # For the speech 
import IRGestureRec         # For interfacing with IR proximity sensor 
import feedparser
from TTSHandler import TTSHandler
import pyupm_grove          
import time
import pyupm_i2clcd
import PinConfig
import configuration
import threading
from MailThread import MailThread
import signal
import sys
import os


class SpeakAssistantApp:
    """
    The main class. Creates an mailthread and supporting objects.
    Also starts the mail thread.
    """
    
    def __init__(self, lcd):
        """
        Constructor, intilazies the stuff that we need
        """
        self.operations = configuration.operations 
        self.speaker = TTSHandler('espeak')
        self.speaker.setSpeed(configuration.ttsSpead)
        self.lock = threading.Lock()
        self.event = threading.Event()
        self.lcd = lcd 
        os.environ['ALSA_CARD'] = str(configuration.soundCard)
        self.mailThread = MailThread(configuration.mailCredentials['userName'], configuration.mailCredentials['pwd'], self.lock, 
            self.event, self.lcd, configuration.senders, configuration.unreadMailCnt)
        try:
            self.mailThread.start()
        except:
            self.mailThread = None
            print "Could not start the mail thread"
            pass
   
             
    def mainLoop(self):
        """
        The "heart" of the application. In an infinite loop, waits for IR events and based on
        detected event, takes required action
        """
        
        while 1:
            usrChoice = IRGestureRec.IRGetCount()
            #try:
            try:
                op = self.operations[usrChoice-1]
            except:
                print "Unknown config value "
                continue             # will pass this exception
                
            print "Op is: "
            print op
            print usrChoice
            self.lcd.clear()
            self.lcd.setCursor(0, 0)
            self.lcd.setColor(20, 20, 128)
            #lcd.write("                ")
            self.lcd.write(op[0])
            msg = ""
            operation = op[0].upper()
            
            if operation == 'RSS':
                self.lock.acquire()
                try:
                    fp = feedparser.parse(op[1])
                except:
                    msg = "Could not get the RSS feed"
                    
                print op[1]
                try:
                    msg  = fp['entries'][0][op[2]]
                except:
                    msg = "Could not get the RSS feed"
                    p

                print msg
                self.lock.release()
                self.speaker.speak(msg.encode('ASCII'))
            elif operation == 'TEMPERATURE':
                temp = pyupm_grove.GroveTemp(op[1])
                temp = temp.value()
                print temp
                self.lcd.setCursor(1, 0)
                self.lcd.write(str(temp))
                self.speaker.speak('Temperature is '+ str(temp) + ' degrees celcius')
                
            elif operation == 'GPIO':
                pin = int(op[2])
                import mraa as pymraa 
                gpioPin = pymraa.Gpio(pin)
                gpioPin.dir(pymraa.DIR_OUT)
                print op[1]
                if op[1] == 'on':
                    print "It is on"
                    gpioPin.write(1)
                else:
                    print "It is off"
                    gpioPin.write(0)
                    
            elif operation == 'MAIL':
                try:
                    mails = self.mailThread.getTheMails()
                except:
                    e = sys.exc_info()[0]
                    print "Error fetching mail"
                    print e
                    continue
                    
                if len(mails) == 0:
                    self.speaker.speak("No mail to read")
                else:
                    for mail in mails:
                        print mail.email
                        print mail.sender
                        print mail.subject
                        self.lcd.clear()
                        try:
                            self.lcd.setCursor(0,0)
                            sender = str(mail.sender).encode('ASCII')
                            self.lcd.write(sender[0:15])
                            self.lcd.setCursor(1, 0)
                            subject = str(mail.subject).encode('ASCII')
                            self.lcd.write(subject[0:15])
                            self.speaker.speak("Sender: " + sender)
                            self.speaker.speak("Subject: " + subject)
                        except:
                            self.lcd.clear()
                            pass
                        
                    
           #except:
            #    pass
    def stopSpeak(self):
        """
        When we need to stop the mail thread and try to exit gracefully.
        NOTE: This method has to be tied to the ctrl+c(aka SIGIINT) handler
        """
        self.lcd.clear()
        self.lcd.write("Bye Bye....")
        self.event.set()
        self.mailThread.join()
        self.lcd.clear()
        
        
        
speak = None


# Define the exit handler
def signalHandler(signal, frame):
        print('Exiting.....')
        if speak != None:
            speak.stopSpeak()
        sys.exit(0)
        
# Register the exit handler        
signal.signal(signal.SIGINT, signalHandler)

if __name__ == '__main__':
    lcd = pyupm_i2clcd.Jhd1313m1(PinConfig.LCD_I2C_PORT, 0x3E, 0x62)
    speak = SpeakAssistantApp(lcd)
    print speak.operations
    speak.mainLoop()
    

