#!/usr/bin/python

#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are
#  met:
#  
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following disclaimer
#    in the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of the  nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#  
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#  



from threading import Thread     # For implementing the mail daemon thread
import gmail                     # For accessing the gmail thread
import datetime
import sys
from LcdBackLightManager import LcdBackLightManager

MAIL_THREAD_NORMAL_DELAY = 10      # Polling delay in normal case
MAIL_THREAD_ERROR_DELAY = 50       # Delay in case of an error
MAIL_THREAD_INVALID_COUNT = -1     # Invalid number of mails

class MailAttrib:
    def __init__(self, sender, email, subject):
        self.sender = sender        # Sender of the mail
        self.email = email          # Email id of the sender 
        self.subject = subject      # Subject of the mail

class MailThread(Thread):
    """ This thread keeps polling for any incoming message from gmail server
    and updates a list given to it with the message headings it
    has recived """
    def __init__(self, usrName, pwd, lock, stopEvent, lcd, senderList=[], unreadMailLimit=50):
        """ The consrtuctor 

        usrName     The gmail account id goes here
        pwd         The password for usrName
        lock        Lock object for sync
        stopEvent   Event to be raised for stopping the fetch thread
        lcd         lcd object using which the count would be displayed
        senderList  List of senders whose mails take prefernce (most preferd come first in list)


        """
        super(MailThread, self).__init__()
        self.senderList = senderList
        self.unreadMsgCount = 0
        self.lock = lock
        self.timeAfter = datetime.datetime.now()
        self.usrName = usrName
        self.pwd = pwd
        self.stopEvent = stopEvent
        self.lcd = lcd
        self.backLight = LcdBackLightManager(self.lcd, unreadMailLimit)


    def run(self):
        """ Keeps track of number of unread mails """
        import time
        
        delay = MAIL_THREAD_NORMAL_DELAY

        while 1:
            if self.stopEvent.isSet():
                return

            self.lock.acquire()
            try:
                print "Trying to get msg..."
                self.gmail = gmail.login(self.usrName, self.pwd)
                mails = self.gmail.inbox().mail(unread=True)
                self.unreadMsgCount = len(mails)
                self.lcd.clear()
                self.lcd.setCursor(0, 0)
                self.lcd.write("Unread msg cnt:")
                self.lcd.setCursor(1, 0)
                temp = str(self.unreadMsgCount)
                x = len(temp)
                msg = temp.rjust(16/2)
                self.lcd.write(msg)
                #self.lcd.write()
                self.backLight.setTemp(self.unreadMsgCount)
                self.gmail.logout()
                print self.unreadMsgCount
            except:
                # Could be network issue, wait longer than usual
                e = sys.exc_info()[0]
                print "Error fetching mail"
                print e
                delay = MAIL_THREAD_ERROR_DELAY
                self.unreadMsgCount = MAIL_THREAD_INVALID_COUNT
                pass

            self.lock.release()
            time.sleep(delay)

    def getMsgCount(self):
        """ Returns number of unread mails in the inbox
        """
        self.lock.acquire()
        x = self.unreadMsgCount
        self.lock.release()
        return x

    def getTheMails(self):
        """ Populates the mail queue with the subject and sender """
        mailList = []
        print "Getting the messages..."
        self.lock.acquire()
        self.gmail = gmail.login(self.usrName, self.pwd)
        if len(self.senderList) > 0:
            for sender in self.senderList:   # Let's walk through sender list and fetch the priority mails
                try:
                    mails = self.gmail.inbox().mail(unread=True, after = self.timeAfter, sender=sender)
                except :
                    pass

                if len(mails) > 0:
                    mailList.extend(mails)
        else:
            try:
                mails = self.gmail.inbox().mail(unread=True, after = self.timeAfter)
                mailList.extend(mails)
            except:
                pass

        # Update the time after so that we only fetch the mails that we need
        self.timeAfter = datetime.datetime.now()
        mails=[]
        
        # Walk through all the valid mail objects and process them
        for mail in mailList:
            try:
                mail.fetch()
                name = mail.fr.split("<")[0]
                mailId = mail.fr.split("<")[1].replace(">", '')
                #print name
                #print mail
                mails.append(MailAttrib(name, mailId, mail.subject))
            except:
                pass
            #print mail.fr
            #print mail.subject

        self.gmail.logout()
        self.lock.release()
        return mails
    
    def join(self, timeout=None):
        super(MailThread, self).join(timeout)

    def setSenderList(self, senderList=[]):
        if (type(senderList) != type([])):
            return
        self.senderList = senderList



import threading
import pyupm_i2clcd

def main():
    lock = threading.Lock()
    event = threading.Event()
    lcd = pyupm_i2clcd.Jhd1313m1(0, 0x3E, 0x62)
    obj = MailThread('YOUR_ID', 'YOUR_PWD', lock, event, lcd, ['navin.bhaskar.5@gmail.com'])
    print "Starting the thread"
    obj.start()

    i=0
    while i<3:
        if obj.getMsgCount() > 0:
            mails = obj.getTheMails()
            for mail in mails:
                print mail.email
                print mail.sender
                print mail.subject
            event.set()
            obj.join()
        else:
            continue
        i = i+1



if __name__ == '__main__':
    main()



